//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Portfolio-Regression - by Transcendreamer"
#property script_show_inputs
#include <Math\Alglib\alglib.mqh>

input double trend_step=100;
input string symbol_name="PORTFOLIO";

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnStart()
  {

   Print("###### PORTFOLIO REGRESSION ######");

//---

   int points=0;
   int assets=0;

   string filenames[];
   double balances[];
   double capital=0;

   struct DATA { double equity[]; datetime time[]; };
   DATA dataset[];

   assets=FileSelectDialog("SELECT EQUITY DATA FILES",NULL,
                           "CSV files (*.csv)|*.csv|All files (*.*)|*.*",
                           FSD_ALLOW_MULTISELECT|FSD_COMMON_FOLDER,filenames);

   if(assets<1)
     {
      Alert("NO DATA FILES SELECTED");
      return;
     }

   ArrayResize(dataset,assets);
   ArrayResize(balances,assets);

   for(int i=0; i<assets; i++)
     {

      int handle=FileOpen(filenames[i],FILE_READ|FILE_IS_CSV|FILE_ANSI|FILE_COMMON,'\\');

      if(handle==INVALID_HANDLE)
        {
         Alert("CAN NOT OPEN FILE: ",filenames[i]);
         continue;
        }

      points=0;

      while(!FileIsEnding(handle))
        {
         string parts[];
         string text=FileReadString(handle);
         int k=StringSplit(text,'\t',parts);

         if(k!=2)
           {
            Alert("FILE FORMAT BROKEN");
            return;
           }

         points++;
         ArrayResize(dataset[i].equity,points,256);
         ArrayResize(dataset[i].time,points,256);

         dataset[i].equity[points-1]=StringToDouble(parts[1]);
         dataset[i].time[points-1]=StringToTime(parts[0]);
        }

      FileClose(handle);

     }

//---

   int datasize[];
   ArrayResize(datasize,assets);

   for(int i=0; i<assets; i++)
      if(datasize[i]!=datasize[0])
        {
         Alert("INCORRECT DATA SIZES");
         return;
        }

   for(int j=0; j<points; j++)
      for(int i=0; i<assets; i++)
         if(dataset[i].time[j]!=dataset[0].time[j])
           {
            Alert("INCORRECT TIME STAMPS");
            return;
           }

   datetime timestamps[];
   ArrayResize(timestamps,points);
   for(int j=0; j<points; j++)
      timestamps[j]=dataset[0].time[j];

//---

   for(int i=0; i<assets; i++)
     {
      balances[i]=dataset[i].equity[0];
      capital+=balances[i];
     }

   for(int i=0; i<assets; i++)
      for(int j=0; j<points; j++)
        {
         dataset[i].equity[j]-=balances[i];
        }

//---

   int info;
   CLinearModelShell LM;
   CLRReportShell AR;
   CMatrixDouble MATRIX(points,assets+1);
   double MODEL[];
   ArrayResize(MODEL,points);

   if(trend_step==0)
     {
      Alert("ZERO TREND STEP");
      return;
     }

   for(int j=0; j<points; j++)
     {
      MODEL[j]=trend_step*j;
     }

   for(int i=0; i<assets; i++)
      for(int j=0; j<points; j++)
         MATRIX[j].Set(i,dataset[i].equity[j]);

   for(int j=0; j<points; j++)
      MATRIX[j].Set(assets,MODEL[j]);

   CAlglib::LRBuildZ(MATRIX,points,assets,info,LM,AR);

   if(info<0)
     {
      Alert("REGRESSION MODEL ERROR");
     }

   double ROOTS[];
   ArrayResize(ROOTS,assets);
   CAlglib::LRUnpack(LM,ROOTS,assets);
   Print("ROOTS:");

   for(int i=0; i<assets; i++)
     {
      Print(ROOTS[i]);
     }

   Print("-------------------------");

//---

   double PORTFOLIO[];
   ArrayResize(PORTFOLIO,points);
   ArrayInitialize(PORTFOLIO,0);

   for(int j=0; j<points; j++)
      for(int i=0; i<assets; i++)
        {
         PORTFOLIO[j]+=dataset[i].equity[j]*ROOTS[i];
        }

   double DEV[];
   ArrayResize(DEV,points);

   for(int j=0; j<points; j++)
      DEV[j]=PORTFOLIO[j]-MODEL[j];

   double ssreg=0;
   double ssres=0;

   for(int j=0; j<points; j++)
      ssreg+=(PORTFOLIO[j]*PORTFOLIO[j]);

   for(int j=0; j<points; j++)
      ssres+=(DEV[j]*DEV[j]);

   double r2=ssreg/(ssreg+ssres);
   Print("R^2=",r2);

//---

   for(int j=0; j<points; j++)
      PORTFOLIO[j]+=capital;

   double change[];
   ArrayResize(change,points);

   for(int j=0; j<points; j++)
      if(j>0)
         change[j]=PORTFOLIO[j]-PORTFOLIO[j-1];
      else
         change[j]=0;

   double mean=0;
   for(int j=0; j<points; j++)
      mean+=change[j];
   mean/=points;

   double stdev=0;
   for(int j=0; j<points; j++)
      stdev+=(change[j]-mean)*(change[j]-mean);
   stdev=MathSqrt(stdev/points);

   Print("MEAN=",mean);
   Print("STDEV=",stdev);
   Print("SHARPE=",mean/stdev);

//---

   double sumprofit=0;
   double sumloss=0;

   for(int j=0; j<points; j++)
      if(change[j]>0)
         sumprofit+=change[j];
      else
         sumloss+=-change[j];

   Print("PROFITS=",sumprofit);
   Print("LOSSES=",sumloss);

   if(sumloss!=0)
      Print("PF=",sumprofit/sumloss);
   else
      Print("PF=INF");

//---

   MqlRates rates[];
   ArrayResize(rates,points);

   long starting=LONG_MAX;
   long ending=LONG_MIN;

   for(int j=0; j<points; j++)
     {
      rates[j].close=PORTFOLIO[j];
      rates[j].open=PORTFOLIO[j];
      rates[j].high=PORTFOLIO[j];
      rates[j].low=PORTFOLIO[j];
      rates[j].time=timestamps[j];
      rates[j].real_volume=0;
      rates[j].tick_volume=0;
      rates[j].spread=0;

      if(timestamps[j]>ending)
         ending=timestamps[j];

      if(timestamps[j]<starting)
         starting=timestamps[j];
     }

//---

   CustomSymbolCreate(symbol_name);
   CustomSymbolSetInteger(symbol_name,SYMBOL_DIGITS,2);

   if(!SymbolSelect(symbol_name,true))
     {
      Alert("CAN NOT FIND SYMBOL: ",symbol_name);
      return;
     }

   datetime time1=D'1984.01.01 00:00';
   datetime time2=D'2077.01.01 00:00';
   CustomRatesDelete(symbol_name,time1,time2);
   
   int k=CustomRatesReplace(symbol_name,starting,ending,rates);
   Print("TOTAL PORTFOLIO POINTS: ",k);

   long id=ChartOpen(symbol_name,PERIOD_D1);
   ChartSetInteger(id,CHART_MODE,CHART_LINE);

//---

   return;

  }

//+------------------------------------------------------------------+
